# Copyright 2021 (c) Anna Schumaker.
import lib
import re
import unittest
from gi.repository import Gtk

class TestRegex(unittest.TestCase):
    def changed(self, filter, how):
        self.how = how

    def test_init(self):
        filter = lib.filter.Regex()
        self.assertIsInstance(filter, Gtk.Filter)
        self.assertIsInstance(filter.pattern, re.Pattern)
        self.assertEqual(filter.pattern.pattern, "")
        with self.assertRaises(NotImplementedError):
            filter.do_match(None)

    def test_set_pattern(self):
        filter = lib.filter.Regex()
        filter.connect("changed", self.changed)

        filter.set_pattern("abc")
        self.assertEqual(filter.pattern.pattern, "abc")
        self.assertEqual(self.how, Gtk.FilterChange.MORE_STRICT)

        filter.set_pattern("ab")
        self.assertEqual(filter.pattern.pattern, "ab")
        self.assertEqual(self.how, Gtk.FilterChange.LESS_STRICT)

        filter.set_pattern("cd")
        self.assertEqual(self.how, Gtk.FilterChange.DIFFERENT)
        self.assertEqual(filter.pattern.pattern, "cd")

    def test_search(self):
        filter = lib.filter.Regex()
        filter.set_pattern("ab")

        self.assertTrue(filter.search("abc"))
        self.assertFalse(filter.search("cde"))


class TestPopover(unittest.TestCase):
    def test_init(self):
        popover = lib.filter.Popover()
        self.assertIsInstance(popover, Gtk.Popover)
        self.assertIsInstance(popover.get_child(), Gtk.Label)
        self.assertFalse(popover.get_autohide())

    def test_popup(self):
        popover = lib.filter.Popover()
        popover.popup("test")
        self.assertEqual(popover.get_child().get_text(), "test")


class TestEntry(unittest.TestCase):
    def test_init(self):
        filter = lib.filter.Regex()
        entry = lib.filter.Entry(filter)

        self.assertIsInstance(entry, Gtk.SearchEntry)
        self.assertIsInstance(entry.popover, lib.filter.Popover)
        self.assertEqual(entry.get_property("placeholder-text"), "Type to filter")
        self.assertEqual(entry.get_margin_top(), 5)
        self.assertEqual(entry.get_margin_bottom(), 5)
        self.assertEqual(entry.get_margin_start(), 5)
        self.assertEqual(entry.get_margin_end(), 5)
        self.assertEqual(entry.filter, filter)
        self.assertEqual(entry.popover.get_parent(), entry)

    def test_changed(self):
        filter = lib.filter.Regex()
        entry = lib.filter.Entry(filter)

        entry.set_text("(abc")
        entry.changed(entry)
        self.assertEqual(filter.pattern.pattern, "")
        self.assertTrue(entry.has_css_class("warning"))

        entry.set_text("(abc)")
        entry.changed(entry)
        self.assertEqual(filter.pattern.pattern, "(abc)")
        self.assertFalse(entry.has_css_class("warning"))
