# Copyright 2021 (c) Anna Schumaker.
from . import metadata
import pathlib
import unittest

test_tracks  = pathlib.Path("./data/Test Album")
track_01 = test_tracks / "01 - Test Track.ogg"
track_02 = test_tracks / "02 - Test {Disc 2}.ogg"

class TestMetadata(unittest.TestCase):
    def test_metadata_init(self):
        mdf = metadata.Metadata(track_01)
        self.assertEqual(mdf.path, track_01)
        self.assertIsNone(mdf.file)

    def test_metadata_track_01(self):
        with metadata.Metadata(track_01) as mdf:
            self.assertEqual(mdf.album(),  "Test Album")
            self.assertEqual(mdf.artist(), "Test Artist")
            self.assertEqual(mdf.artistsort(), "Artist, Test")
            self.assertEqual(mdf.decade(),  2010)
            self.assertEqual(mdf.discnumber(), 1)
            self.assertEqual(mdf.genres(), [ "Test" ])
            self.assertEqual(mdf.length(), 10)
            self.assertEqual(mdf.title(), "Test Track")
            self.assertEqual(mdf.tracknumber(), 1)
            self.assertEqual(mdf.year(), 2019)

    def test_metadata_track_02(self):
        with metadata.Metadata(track_02) as mdf:
            self.assertEqual(mdf.artist(), "Test Album Artist")
            self.assertEqual(mdf.artistsort(), "Album Artist, Test")
            self.assertEqual(mdf.genres(), [ "Test", "Genre", "List" ])
            self.assertEqual(mdf.year(), 2019)
